#define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
#include "cluster_reader.h" //this is the function that we want to call
#include <Python.h>
#include <numpy/arrayobject.h>
#include<iostream>
using namespace std;


// Docstring
static char module_docstring[] = "Cluster reader";


PyDoc_STRVAR(read_clusters_doc, "Read clusters from open file.\n\n"
                      "Parameters\n"
                      "----------\n"
                      "f: file name\n"

                      "Returns\n"
                      "----------\n"
                      "nothing\n\n");




PyDoc_STRVAR(open_cluster_file_doc, "Read cluster file.\n\n"
                      "Parameters\n"
                      "----------\n"
                      "maxframes: maximum number of frames to be read\n"

                      "Returns\n"
                      "----------\n"
                      "v: numpy_array\n"
                      "   clusters\n\n");


PyDoc_STRVAR(close_cluster_file_doc, "Read cluster file.\n\n"
                      "Parameters\n"
                      "----------\n"
                      "none\n"
                      "Returns\n"
                      "----------\n"
                      "nothing\n\n");


// Declare functions that should go into the module
static PyObject *read_clusters(PyObject *self, PyObject *args);
static PyObject *open_cluster_file(PyObject *self, PyObject *args);
static PyObject *close_cluster_file(PyObject *self, PyObject *args);

// This is the module itself
static PyMethodDef module_methods[] = {{"open_cluster_file", (PyCFunction)open_cluster_file, METH_VARARGS, open_cluster_file_doc},{"close_cluster_file", (PyCFunction)close_cluster_file, METH_VARARGS, close_cluster_file_doc},{"read_clusters", (PyCFunction)read_clusters, METH_VARARGS, read_clusters_doc},
    {NULL, NULL, 0, NULL}};

// Don't touch except the name
static struct PyModuleDef cluster_reader_module_def = {PyModuleDef_HEAD_INIT, "cluster_reader_module",
                                       module_docstring, -1, module_methods};

// Don't touch
PyMODINIT_FUNC PyInit_cluster_reader_cpp(void) {
    PyObject *m = PyModule_Create(&cluster_reader_module_def);
    if (m == NULL)
        return NULL;

    import_array();
    return m;
}

static PyObject *open_cluster_file(PyObject *self, PyObject *args) {

//    cout << "Hello there" << endl; // General Kenobi
    // Python offers a lot of flexibility so the first thing we need to do
    // is to parse the arguments and make sure we are called with correct
    // parameters
  char *fname="";
    if (!PyArg_ParseTuple(args, "s", fname))
        return NULL;

    // Now call add wih pointers and size
    int ok;
    cpp_open_cluster_file(fname,ok);

    return Py_BuildValue("i",ok);
}

static PyObject *close_cluster_file(PyObject *self, PyObject *args) {

//    cout << "Hello there" << endl; // General Kenobi
    // Python offers a lot of flexibility so the first thing we need to do
    // is to parse the arguments and make sure we are called with correct
    // parameters
 
    // Now call add wih pointers and size
    int ok;
    cpp_close_cluster_file(ok);

    return Py_BuildValue("i",ok);
}


static PyObject *read_clusters(PyObject *self, PyObject *args) {

//    cout << "Hello there" << endl; // General Kenobi
    // Python offers a lot of flexibility so the first thing we need to do
    // is to parse the arguments and make sure we are called with correct
    // parameters
    PyObject *clust_obj;
    //PyObject *maxframes_obj;
    if (!PyArg_ParseTuple(args, "O", &clust_obj))
        return NULL;

    // Create two numpy arrays from the passed objects, if possible numpy will
    // use the underlying buffer, otherwise it will create a copy, for example
    // if data type is different or we pass in a list. The
    // NPY_ARRAY_C_CONTIGUOUS flag ensures that we have contiguous memory.
    PyObject *clust_array =
        PyArray_FROM_OTF(clust_obj, NPY_OBJECT, NPY_ARRAY_C_CONTIGUOUS);
    // npy_intp ind=0;
    // void* clust_ptr = PyArray_GetPtr((PyArrayObject*)clust_obj, &ind);
    
      // If parsing of a or b fails we throw an exception in Python
    if (clust_array == NULL) {
        PyErr_SetString(
            PyExc_TypeError,
            "Could not convert one of the arguments to a numpy array.");
        return NULL;
    }
    int N=MAXPH;
    //  int N = (int)PyArray_DIM(clust_array, 0);
    //npy_intp dims[2] = { 400, 400 };

    // Create array for return values
  
    // For the C++ function call we need pointers (or another C++ type/data
    // structure)
    struct cluster *clust = reinterpret_cast<struct cluster *>(
        PyArray_DATA(reinterpret_cast<PyArrayObject *>(clust_array)));

    int nph, nframes,ok;
    // Now call add wih pointers and size
    cpp_read_clusters(clust,nph,nframes,ok,N);

    // Clean up
    Py_DECREF(clust_array);

//    return result;
    return Py_BuildValue("iii",nph,nframes,ok);
}
